
precision highp float;

#define MAX_TEXTURES 2

uniform int u_cc3Texture2DCount;
uniform sampler2D s_cc3Texture2Ds[MAX_TEXTURES];

varying vec3 v_N;
varying vec3 v_T;
varying vec3 v_B;
varying vec2 v_texCoord;


float hue_to_rgb(float v1, float v2, float vH) {
   if (vH < 0.0) vH += 1.0;
   if (vH > 1.0) vH -= 1.0;
   if ((6.0 * vH) < 1.0) return v1 + (v2 - v1) * 6.0 * vH;
   if ((2.0 * vH) < 1.0) return v2;
   if ((3.0 * vH) < 2.0) return v1 + (v2 - v1) * ((2.0 / 3.0) - vH) * 6.0;
   return v1;
}

vec3 rgb_to_hsl(vec3 rgb) {
    float r = rgb.x;
    float g = rgb.y;
    float b = rgb.z;

    float min_color = min(r, min(g, b));
    float max_color = max(r, max(g, b));
    float del_max = max_color - min_color;

    float h = 0.0;
    float s = 0.0;
    float l = (max_color + min_color) / 2.0;
    
    if (del_max != 0.0) {
        if (l < 0.5) {
       	    s = del_max / (max_color + min_color);
        } else {
       	    s = del_max / (2.0 - max_color - min_color);
        }
        float del_r = (((max_color - r) / 6.0) + max_color / 2.0) / max_color;
        float del_g = (((max_color - g) / 6.0) + max_color / 2.0) / max_color;
        float del_b = (((max_color - b) / 6.0) + max_color / 2.0) / max_color;
        if (r == max_color) {
       	    h = del_b - del_g;
	} else if (g == max_color) {
	    h = 1.0 / 3.0 + del_r - del_b;
        } else if (b == max_color) {
	    h = 2.0 / 3.0 + del_g - del_r;
        }
	if (h < 0.0) {
	    h += 1.0;
	}
	if (h > 1.0) {
	    h -= 1.0;
	}
    }
    return vec3(h, s, l);
}

vec3 hsl_to_rgb(vec3 hsl) {
    float h = hsl.x;
    float s = hsl.y;
    float l = hsl.z;
    float r;
    float g;
    float b;
    if (s == 0.0) {
        r = l;
	g = l;
	b = l;
    } else {
      	float var2;
        if (l < 0.5) {
	    var2 = l * (1.0 + s);
	} else {
	    var2 = l + s - s * l;
	}
	float var1 = 2.0 * l - var2;
	r = hue_to_rgb(var1, var2, h + 1.0 / 3.0);
	g = hue_to_rgb(var1, var2, h);
	b = hue_to_rgb(var1, var2, h - 1.0 / 3.0);
    }
    return vec3(r, g, b);
}

void main() {
    vec4 texel = texture2D(s_cc3Texture2Ds[0], v_texCoord);
    vec4 normal = texture2D(s_cc3Texture2Ds[1], v_texCoord);

    if (texel.a < 0.1) {
        discard;
    }

    vec3 N;
    if (u_cc3Texture2DCount == 1) {
        N = v_N;
    } else {
        vec3 N_t = normal.rgb * 2.0 - 1.0;
        N = normalize(mat3(v_T, v_B, v_N) * N_t);
    }

    vec4 pre_color = texel;

    vec3 hsl = rgb_to_hsl(vec3(pre_color.x, pre_color.y, pre_color.z));
    vec3 blue_hsl = vec3(0.580702, 0.852941, 0.66); // was this: rgb_to_hsl(vec3(0.37, 0.86, 0.95));
    vec3 new_rgb = hsl_to_rgb(vec3(blue_hsl.x, blue_hsl.y, hsl.z));

    gl_FragColor = vec4(mix(pre_color.rgb, new_rgb, pre_color.w), pre_color.w);
}
