//#define EGG_PROGRESS

precision highp float;

// uniforms
uniform sampler2D s_cc3Texture2Ds[2];
uniform vec3 u_cc3CameraPositionGlobal;
uniform vec2 u_cc3SceneTime;
uniform float pulsePhase;
uniform vec4 dyeColor;
uniform vec4 glowColor;
uniform float percentageDone;
uniform float tint;
uniform float u_cc3ModelBoundingRadius;

// varyings
varying vec3 v_position_world;
varying vec2 v_texCoord;
varying vec3 v_position_local;

// HSV
vec3 rgb2hsv(vec3 c) {
    vec4 K = vec4(0.0, -1.0 / 3.0, 2.0 / 3.0, -1.0);
    vec4 p = mix(vec4(c.bg, K.wz), vec4(c.gb, K.xy), step(c.b, c.g));
    vec4 q = mix(vec4(p.xyw, c.r), vec4(c.r, p.yzx), step(p.x, c.r));

    float d = q.x - min(q.w, q.y);
    float e = 1.0e-10;
    return vec3(abs(q.z + (q.w - q.y) / (6.0 * d + e)), d / (q.x + e), q.x);
}

vec3 hsv2rgb(vec3 c) {
    vec4 K = vec4(1.0, 2.0 / 3.0, 1.0 / 3.0, 3.0);
    vec3 p = abs(fract(c.xxx + K.xyz) * 6.0 - K.www);
    return c.z * mix(K.xxx, clamp(p - K.xxx, 0.0, 1.0), c.y);
}

// code
void main() {
    vec4 texel1 = texture2D(s_cc3Texture2Ds[0], v_texCoord);
    vec4 texel2 = texture2D(s_cc3Texture2Ds[1], v_texCoord);

    float time = mod(u_cc3SceneTime.x, pulsePhase);

    const float PI = 3.1415926535;
    float glowIntensity = sin(PI * time / pulsePhase);
    glowIntensity *= glowIntensity;

    vec3 colorBeforeGlowHSV = rgb2hsv(texel1.rgb);
    vec3 dyeColorHSV = rgb2hsv(dyeColor.rgb);

    float afterDyeV = float(dyeColorHSV.b < 0.5) * mix(0.0, colorBeforeGlowHSV.b, 2.0 * dyeColorHSV.b) +
        float(dyeColorHSV.b >= 0.5) * mix(colorBeforeGlowHSV.b, 1.0, 2.0 * dyeColorHSV.b - 1.0);

    vec3 colorAfterDyeHSV = vec3(dyeColorHSV.r, dyeColorHSV.g, afterDyeV);
    vec3 colorAfterDye = hsv2rgb(colorAfterDyeHSV);

    vec3 colorAfterGlow = mix(texel1.rgb, colorAfterDye, texel1.a) + glowIntensity * texel2.a * glowColor.rgb;
    
    // grayed out part
    vec3 progressOffsetColor = vec3(0.0, 0.0, 0.0);
    vec3 grayedOutTextureColorRGB = mix(texel1.rgb, progressOffsetColor, texel2.a);
    vec3 grayedOutTextureColorHSV = rgb2hsv(grayedOutTextureColorRGB);
    float valueDivider = 2.0;
    vec3 grayedOutTextureColorHSVPlus = vec3(grayedOutTextureColorHSV.r, grayedOutTextureColorHSV.g, grayedOutTextureColorHSV.b / valueDivider);
    vec3 finalGrayedOut = hsv2rgb(grayedOutTextureColorHSVPlus);
    
#ifdef EGG_PROGRESS
    float inDiscoveredPart = float(v_position_local.y >= 2.0 * u_cc3ModelBoundingRadius * percentageDone);
#else
    float inDiscoveredPart = float(v_position_local.y >= 2.0 * u_cc3ModelBoundingRadius);
#endif
    
    
    
    // no ifs allowed
    vec3 finalColor = vec3(inDiscoveredPart * finalGrayedOut.r + (1.0 - inDiscoveredPart) * colorAfterGlow.r,
                           inDiscoveredPart * finalGrayedOut.g + (1.0 - inDiscoveredPart) * colorAfterGlow.g,
                           inDiscoveredPart * finalGrayedOut.b + (1.0 - inDiscoveredPart) * colorAfterGlow.b);
    finalColor *= tint;

    gl_FragColor = vec4(finalColor, 1.0);
}
