precision highp float;

attribute vec4 a_cc3Position;
attribute vec4 a_cc3Normal;
attribute vec4 a_cc3Color;
attribute vec2 a_cc3TexCoord0;
attribute vec2 a_cc3TexCoord1;

// uniforms
uniform mat4 u_cc3MatrixModel;
uniform mat4 u_cc3MatrixView;
uniform mat4 u_cc3MatrixProj;
uniform vec2 u_cc3SceneTime;

uniform float waveMaxHeight;

uniform float waveLength;
uniform float waveFrequency;

uniform float verticalFrequency;
uniform float verticalWeight;

// varyings
varying vec2 v_texCoord;
varying vec2 v_texCoord1;

#include "shadow.vsh"

void main() {
    v_texCoord = a_cc3TexCoord0;
    v_texCoord1 = a_cc3TexCoord1;

    vec4 position_model = a_cc3Position;

    float waveHeight = mix(0.0, waveMaxHeight, a_cc3Color.r);

    float dist = position_model.x;
    float phase = waveLength * dist - waveFrequency * u_cc3SceneTime[0];

    position_model.y -= verticalWeight * cos(verticalFrequency * u_cc3SceneTime[0]) * a_cc3Color.r;
    position_model.z += waveHeight * cos(phase);


    vec4 position_world = u_cc3MatrixModel * position_model;

    gl_Position = u_cc3MatrixProj * u_cc3MatrixView * position_world;

    shadow_receive(position_world);
}
