
precision highp float;

uniform sampler2D s_cc3Texture2Ds[2];
uniform samplerCube s_cc3TextureCube;
uniform vec3 u_cc3CameraPositionGlobal;
uniform vec2 u_cc3SceneTime;

uniform float reflectionLevel;
uniform vec2 speed;

varying vec3 v_position_world;
varying vec2 v_texCoord;

void main() {
    float time = mod(u_cc3SceneTime[0], 1.0 / max(abs(speed.x), abs(speed.y)));

    // use the normal map to get a smooth displacement for both diffuse and wrinkles
    vec2 deltaCoords = v_texCoord * 0.2 + speed * time;
    vec2 delta = texture2D(s_cc3Texture2Ds[1], deltaCoords).xy * 2.0 - 1.0;
    vec2 displacedCoords = v_texCoord * 0.6 + delta * 0.02 + speed * time;

    vec4 diffuseTexel = texture2D(s_cc3Texture2Ds[0], displacedCoords);
    vec4 normalTexel = texture2D(s_cc3Texture2Ds[1], displacedCoords);
    float glossiness = normalTexel.a;

    // scale X and Y components to avoid reflection vector to go below Y=0
    vec3 N = normalize((normalTexel.xyz * vec3(0.8, 0.8, 2.0) - vec3(0.4, 0.4, 1.0)));
    vec3 V = u_cc3CameraPositionGlobal - v_position_world;
    vec3 R = reflect(-V, vec3(N.x, N.z, -N.y)); // normal must be rotated -90 around X
    vec4 reflection = textureCube(s_cc3TextureCube, R);

    gl_FragColor = mix(diffuseTexel, reflection, reflectionLevel * glossiness);
}
