//#define USE_NORMAL_MAP

#define MAX_LIGHTS 8

precision highp float;

// uniforms
uniform vec4 u_cc3MaterialDiffuseColor;

uniform sampler2D s_cc3Texture2Ds[2];

uniform bool u_cc3LightIsEnabled[MAX_LIGHTS];
uniform vec4 u_cc3LightPositionGlobal[MAX_LIGHTS];
uniform vec4 u_cc3LightDiffuseColor[MAX_LIGHTS];
uniform float u_cc3LightSpotExponent[MAX_LIGHTS];
uniform vec3 u_cc3CameraPositionGlobal;

// varyings
varying vec3 v_position_world;

varying vec3 v_N;
#if defined(USE_NORMAL_MAP)
varying vec3 v_T;
varying vec3 v_B;
#endif

varying vec2 v_texCoord;

// code
vec3 thresholdedValue(float baseValue, float threshold, vec3 smaller, vec3 bigger) {
    return float(baseValue < threshold) * smaller + float(baseValue >= threshold) * bigger;
}

void main() {
    vec4 textureColor = texture2D(s_cc3Texture2Ds[0], v_texCoord);

    // normal
#ifdef USE_NORMAL_MAP
    vec4 normalSpec = texture2D(s_cc3Texture2Ds[1], v_texCoord);
    float specLevel = normalSpec.a;
    vec3 normal_t = normalSpec.rgb * 2.0 - 1.0;
    vec3 N = normalize(mat3(v_T, v_B, v_N) * normal_t);
    N = thresholdedValue(length(N), 1e-3, vec3(0, 0, 1), N);
//    N = v_N;
#else
    float specLevel = 1.0;
    vec3 N = v_N;
#endif
//    gl_FragColor = vec4(N, 1.0); return;
    vec3 totalDiffuseLight = vec3(0);
    vec3 totalSpecularLight = vec3(0);
    for (int i=0; i < MAX_LIGHTS; ++i) {
        if (!u_cc3LightIsEnabled[i]) {
            continue;
        }
        // diffuse
        vec3 lightColor = u_cc3LightDiffuseColor[i].rgb;
        float isPointLight = u_cc3LightPositionGlobal[i].w;
        vec3 L = normalize(u_cc3LightPositionGlobal[i].xyz - isPointLight * v_position_world);
        float nDotL = max(0.0, dot(N, L));
        totalDiffuseLight += nDotL * lightColor;
#define V3
#if defined(V1)
        // specular
        vec3 V = normalize(u_cc3CameraPositionGlobal - v_position_world);
        vec3 H = normalize(L + V);
        float nDotH = max(0.0, dot(N, H));
        float specExponent = u_cc3LightSpotExponent[i];
        totalSpecularLight += pow(nDotH, specExponent);
#elif defined(V2)
        // tests
        vec3 V = normalize(u_cc3CameraPositionGlobal - v_position_world);
        vec3 R = reflect(-L, N);
        float rDotV = max(0.0, dot(R, V));
        float specExponent = u_cc3LightSpotExponent[i];
        totalSpecularLight += pow(rDotV, specExponent);
#elif defined(V3)
        // tests
        vec3 V = normalize(u_cc3CameraPositionGlobal - v_position_world);
        vec3 R = reflect(-V, N);
        float rDotL = max(0.0, dot(R, L));
        float specExponent = u_cc3LightSpotExponent[i];
        totalSpecularLight += pow(rDotL, specExponent);
#endif
    }
    totalDiffuseLight = clamp(totalDiffuseLight, 0, 1);
    totalSpecularLight = clamp(totalSpecularLight, 0, 1);
//    gl_FragColor = vec4(specLevel * totalSpecularLight, 1.0); return;
//    gl_FragColor = vec4(totalDiffuseLight, 1.0); return;
    gl_FragColor = textureColor * u_cc3MaterialDiffuseColor * vec4(totalDiffuseLight, 1.0) + vec4(specLevel * totalSpecularLight, 1.0);
}
