
#if (CC3_PLATFORM_ANDROID == 0)

#if CC3_PLATFORM_WINDOWS // win32

#define INSTANCE_ID 0

#elif defined(GL_ES) // ios

#extension GL_EXT_draw_instanced: enable
#define INSTANCE_ID gl_InstanceIDEXT
#define INSTANCING_ENABLED

#else // osx

#extension GL_ARB_draw_instanced: enable
#define INSTANCE_ID gl_InstanceIDARB

#endif

#endif

//#define #ifdef LIGHTMAP_ENABLED
//#define #ifdef ANIMATING_ENABLED
//#define #ifdef NO_LOOP_TIME

// attributes
precision highp float;

attribute vec4 a_cc3Position;
attribute vec3 a_cc3Normal;
attribute vec3 a_cc3Tangent;
attribute vec3 a_cc3Bitangent;
attribute vec2 a_cc3TexCoord0;
attribute vec4 a_cc3Color;

#ifdef LIGHTMAP_ENABLED
attribute vec2 a_cc3TexCoord1;
#endif

#define MAX_STATIC_INSTANCES 16

// uniforms
#ifdef INSTANCING_ENABLED
    uniform mat4 u_cc3MatricesModel[MAX_STATIC_INSTANCES];
#else
    uniform mat4 u_cc3MatrixModel;
#endif

uniform mat4 u_cc3MatrixView;
uniform mat4 u_cc3MatrixProj;
uniform float u_cc3ModelBoundingRadius;

#ifdef ANIMATING_ENABLED
uniform vec2 speed;
uniform vec2 u_cc3SceneTime;
#ifdef NO_LOOP_TIME
uniform float u_cc3ObjectTime;
uniform float timeOffset;
#endif
#endif

// varyings
varying vec3 v_position_world;
varying vec3 v_position_local;
varying vec2 v_texCoord;

#if defined(USE_LIGHT) || defined(FRESNEL_ENABLED) || defined(BLUE)
varying vec3 v_N;
#if defined(USE_NORMAL_MAP) || defined(BLUE)
varying vec3 v_T;
varying vec3 v_B;
#endif
#endif

#ifdef LIGHTMAP_ENABLED
varying vec2 v_lightMapCoord;
#endif
varying vec4 v_vertexColor;

#include "shadow.vsh"

void main() {
    #ifdef INSTANCING_ENABLED
        mat4 modelMatrix = u_cc3MatricesModel[INSTANCE_ID];
    #else
        mat4 modelMatrix = u_cc3MatrixModel;
    #endif

    vec4 position_world = modelMatrix * a_cc3Position;

    gl_Position = u_cc3MatrixProj * u_cc3MatrixView * position_world;

    v_position_world = position_world.xyz;
    v_position_local = a_cc3Position.xyz;
    v_texCoord = a_cc3TexCoord0;
    v_vertexColor = a_cc3Color;

#if defined(USE_LIGHT) || defined(FRESNEL_ENABLED) || defined(BLUE)
    v_N = normalize(mat3(modelMatrix) * a_cc3Normal);
#if defined(USE_NORMAL_MAP) || defined(BLUE)
    v_T = normalize(mat3(modelMatrix) * a_cc3Tangent);
    v_B = normalize(mat3(modelMatrix) * a_cc3Bitangent);
#endif
#endif

#ifdef LIGHTMAP_ENABLED
    v_lightMapCoord = a_cc3TexCoord1;
#endif
    
#ifdef ANIMATING_ENABLED
#ifdef NO_LOOP_TIME
    float time = timeOffset + u_cc3ObjectTime;
#else
    float time = mod(u_cc3SceneTime[0], 1.0 / max(abs(speed.x), abs(speed.y)));
#endif
    v_texCoord = v_texCoord + time * speed;
#endif

    shadow_receive(position_world);
}
