precision highp float;

// uniforms
uniform vec4 u_cc3MaterialDiffuseColor;
uniform sampler2D s_cc3Texture2Ds[2];

//custom
uniform vec4 flagInsideColor;
uniform vec4 flagOutsideColor;
uniform vec4 emblemDiffuseColor;
uniform float emblemRowNum;
uniform float emblemColumnNum;
uniform float emblemIndex;

// varyings
// flag greyscale and masks
varying vec2 v_texCoord;
// emblem atlas
varying vec2 v_texCoord1;

float overlayFloat(float target, float blend) {
    return float(target > 0.5) * (1.0 - (1.0 - 2.0 * (target - 0.5)) * (1.0 - blend)) + float(target <= 0.5) * ((2.0 * target) * blend);
}

//add color on greyScaleColor
vec3 overlayColor(float greyScaleValue, vec3 color) {
    return vec3(overlayFloat(greyScaleValue, color.r), overlayFloat(greyScaleValue, color.g), overlayFloat(greyScaleValue, color.b));
}

#include "shadow.fsh"

void main() {
    //gl_FragColor = vec4(v_texCoord1, 0, 1); return;
    vec4 flagColor = texture2D(s_cc3Texture2Ds[0], v_texCoord);
    float flagGreyScaleColor = flagColor.r;
    float flagBorderMask = flagColor.g;
    float flagBackgroundMask = flagColor.b;
    float flagAlphaMast = flagColor.a;
    //draw flag
    vec4 flagBorderColor = vec4(overlayColor(flagGreyScaleColor, flagOutsideColor.rgb), flagAlphaMast) * flagBorderMask;
    vec4 flagBackgroundColor = vec4(overlayColor(flagGreyScaleColor, flagInsideColor.rgb), flagAlphaMast) * flagBackgroundMask;
    vec4 finalFlagColor = flagBorderColor + flagBackgroundColor;
    //draw emblem
    float emblemWitdh = 1.0 / emblemColumnNum;
    float emblemHeight = 1.0 / emblemRowNum;
    //map the uv to the correct one on atlas, depending on the index
    vec2 emblemCoord = vec2(mod(emblemIndex, emblemColumnNum) * emblemWitdh + v_texCoord1.x / emblemColumnNum,
                            (emblemRowNum - 1.0 - floor(emblemIndex / emblemRowNum)) * emblemHeight + v_texCoord1.y / emblemRowNum);
    vec4 flagEmblemColor = texture2D(s_cc3Texture2Ds[1], emblemCoord) * emblemDiffuseColor;
    finalFlagColor = finalFlagColor * (1.0 - flagEmblemColor.a) + flagEmblemColor * flagEmblemColor.a * flagAlphaMast;
    //alpha test
    if (finalFlagColor.a < 0.1) {
        discard;
    }
    gl_FragColor = u_cc3MaterialDiffuseColor * finalFlagColor;
    shadow_receive();
}
