
//#define NUM_TEXTURES 2
//#define SPECULAR_ENABLED

precision highp float;

// uniforms
uniform vec4 u_cc3MaterialDiffuseColor;
uniform vec4 u_cc3MaterialSpecularColor; // this is actually the color of the fresnel reflection

uniform sampler2D s_cc3Texture2Ds[NUM_TEXTURES + 2];
uniform samplerCube u_reflectionMap;

uniform vec3 u_cc3CameraPositionGlobal;
uniform vec2 u_cc3SceneTime;

// varyings
varying vec2 v_texCoord;
varying vec2 v_lightMapCoord;

#ifdef SPECULAR_ENABLED
varying vec3 v_N;
varying vec3 v_position_world;
#endif

#ifdef USE_VERTEX_COLOR
varying vec4 v_vertexColor;
#endif

#include "shadow.fsh"

// code
void main() {
    vec4 compositeTexel = texture2D(s_cc3Texture2Ds[NUM_TEXTURES], v_lightMapCoord);

    gl_FragColor = vec4(0, 0, 0, 1);

    // at least 2 diffuse texture
    vec4 diffuse0 = texture2D(s_cc3Texture2Ds[0], v_texCoord);
    vec4 diffuse1 = texture2D(s_cc3Texture2Ds[1], v_texCoord);
    gl_FragColor.rgb += diffuse0.rgb * compositeTexel.r;
    gl_FragColor.rgb += diffuse1.rgb * compositeTexel.g;

    // additional diffuse textures
#if NUM_TEXTURES > 2
    vec4 diffuse2 = texture2D(s_cc3Texture2Ds[2], v_texCoord);
    gl_FragColor.rgb += diffuse2.rgb * compositeTexel.b;
#else
    gl_FragColor.rgb += diffuse0.rgb * compositeTexel.b;
#endif
#if NUM_TEXTURES > 3
    vec4 diffuse3 = texture2D(s_cc3Texture2Ds[3], v_texCoord);
    gl_FragColor.rgb += diffuse3.rgb * compositeTexel.a;
#else
#ifdef ALPHA
    gl_FragColor.rgb += diffuse0.rgb * compositeTexel.a;
#endif
#endif
    

#ifdef USE_VERTEX_COLOR
    gl_FragColor *= v_vertexColor;
#endif

#ifdef SPECULAR_ENABLED
    vec3 N = v_N;
    vec3 V = normalize(u_cc3CameraPositionGlobal - v_position_world);
    vec3 R = reflect(-V, N);
    float specularLevel = diffuse0.a * compositeTexel.r + diffuse1.a * compositeTexel.g;
#if NUM_TEXTURES > 2
    specularLevel += diffuse2.a * compositeTexel.b;
#endif
#if NUM_TEXTURES > 3
    specularLevel += diffuse3.a * compositeTexel.a;
#endif
    vec3 specularColor = specularLevel * textureCube(u_reflectionMap, R).rgb;
    gl_FragColor.rgb += specularColor;
#endif

    // light map
    gl_FragColor *= texture2D(s_cc3Texture2Ds[NUM_TEXTURES+1], v_lightMapCoord);

    shadow_receive();
}
